
{******************************************}
{                                          }
{             FastReport v3.0              }
{       SQLMemTable enduser components     }
{                                          }

// Copyright (c) AidAim Software
// http://www.aidaim.com
// 27 March 2007

{                                          }
{******************************************}

unit frxSMTComponents;

interface

{$I frx.inc}

uses
  Windows, Classes, frxClass, frxCustomDB, DB, SQLMemMain, SQLMemConst
{$IFDEF Delphi6}
, Variants
{$ENDIF}
{$IFDEF QBUILDER}
, fqbClass
{$ENDIF};


type
  TfrxSQLMemComponents = class(TfrxDBComponents)
  private
    FDefaultDatabase: TSQLMemDatabase;
    FOldComponents: TfrxSQLMemComponents;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function GetDescription: String; override;
  published
    property DefaultDatabase: TSQLMemDatabase read FDefaultDatabase write FDefaultDatabase;
  end;

  TfrxSQLMemDatabase = class(TfrxCustomDatabase)
  private
    FDatabase: TSQLMemDatabase;
  protected
    procedure SetConnected(Value: Boolean); override;
    procedure SetDatabaseName(const Value: String); override;
    procedure SetLoginPrompt(Value: Boolean); override;
    procedure SetParams(Value: TStrings); override;
    function GetConnected: Boolean; override;
    function GetDatabaseName: String; override;
    function GetLoginPrompt: Boolean; override;
    function GetParams: TStrings; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    class function GetDescription: String; override;
    procedure SetLogin(const Login, Password: String); override;
    property Database: TSQLMemDatabase read FDatabase;
  published
    property DatabaseName;
    property LoginPrompt;
    property Params;
    property Connected;
  end;

  TfrxSQLMemTable = class(TfrxCustomTable)
  private
    FDatabase: TfrxSQLMemDatabase;
    FTable: TSQLMemTable;
    procedure SetDatabase(const Value: TfrxSQLMemDatabase);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure SetMaster(const Value: TDataSource); override;
    procedure SetMasterFields(const Value: String); override;
    procedure SetIndexFieldNames(const Value: String); override;
    procedure SetIndexName(const Value: String); override;
    procedure SetTableName(const Value: String); override;
    function GetIndexFieldNames: String; override;
    function GetIndexName: String; override;
    function GetTableName: String; override;
  public
    constructor Create(AOwner: TComponent); override;
    constructor DesignCreate(AOwner: TComponent; Flags: Word); override;
    class function GetDescription: String; override;
    procedure BeforeStartReport; override;
    property Table: TSQLMemTable read FTable;
  published
    property Database: TfrxSQLMemDatabase read FDatabase write SetDatabase;
  end;

  TfrxSQLMemQuery = class(TfrxCustomQuery)
  private
    FDatabase: TfrxSQLMemDatabase;
    FQuery: TSQLMemQuery;
    procedure SetDatabase(const Value: TfrxSQLMemDatabase);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure SetMaster(const Value: TDataSource); override;
    procedure SetSQL(Value: TStrings); override;
    function GetSQL: TStrings; override;
  public
    constructor Create(AOwner: TComponent); override;
    constructor DesignCreate(AOwner: TComponent; Flags: Word); override;
    class function GetDescription: String; override;
    procedure BeforeStartReport; override;
    procedure UpdateParams; override;
{$IFDEF QBUILDER}
    function QBEngine: TfqbEngine; override;
{$ENDIF}
    property Query: TSQLMemQuery read FQuery;
  published
    property Database: TfrxSQLMemDatabase read FDatabase write SetDatabase;
  end;

{$IFDEF QBUILDER}
  TfrxEngineSMT = class(TfqbEngine)
  private
    FQuery: TSQLMemQuery;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure ReadTableList(ATableList: TStrings); override;
    procedure ReadFieldList(const ATableName: string; var AFieldList: TfqbFieldList); override;
    function ResultDataSet: TDataSet; override;
    procedure SetSQL(const Value: string); override;
  end;
{$ENDIF}


var
  SMTComponents: TfrxSQLMemComponents;


implementation

{$R *.res}

uses 
  frxSMTRTTI,
{$IFNDEF NO_EDITORS}
  frxSMTEditor, 
{$ENDIF}
  frxDsgnIntf, frxRes;


{ TfrxSQLMemComponents }

constructor TfrxSQLMemComponents.Create(AOwner: TComponent);
begin
  inherited;
  FOldComponents := SMTComponents;
  SMTComponents := Self;
  FDefaultDatabase := TSQLMemDatabase.Create(nil);
  FDefaultDatabase.DatabaseName := SQLMemMemoryDatabaseName;
end;

destructor TfrxSQLMemComponents.Destroy;
begin
  if SMTComponents = Self then
    SMTComponents := FOldComponents;
  FDefaultDatabase.Free; 
  inherited;
end;

function TfrxSQLMemComponents.GetDescription: String;
begin
  Result := 'SMT';
end;


{ TfrxSQLMemDatabase }

constructor TfrxSQLMemDatabase.Create(AOwner: TComponent);
begin
  inherited;
  FDatabase := TSQLMemDatabase.Create(nil);
  Component := FDatabase;
end;

destructor TfrxSQLMemDatabase.Destroy;
begin
  inherited;
end;

class function TfrxSQLMemDatabase.GetDescription: String;
begin
  Result := 'SMT Database';
end;

function TfrxSQLMemDatabase.GetConnected: Boolean;
begin
  Result := FDatabase.Connected;
end;

function TfrxSQLMemDatabase.GetDatabaseName: String;
begin
  Result := FDatabase.DatabaseName;
end;

function TfrxSQLMemDatabase.GetLoginPrompt: Boolean;
begin
  Result := False;
//  Result := FDatabase.LoginPrompt;
end;

function TfrxSQLMemDatabase.GetParams: TStrings;
begin
  Result := nil;
  //Result := FDatabase.Params;
end;

procedure TfrxSQLMemDatabase.SetConnected(Value: Boolean);
begin
  BeforeConnect(Value);
  FDatabase.Connected := Value;
end;

procedure TfrxSQLMemDatabase.SetDatabaseName(const Value: String);
begin
  FDatabase.DatabaseName := Value;
end;

procedure TfrxSQLMemDatabase.SetLoginPrompt(Value: Boolean);
begin
//  FDatabase.LoginPrompt := Value;
end;

procedure TfrxSQLMemDatabase.SetParams(Value: TStrings);
begin
//  FDatabase.Params := Value;
end;

procedure TfrxSQLMemDatabase.SetLogin(const Login, Password: String);
begin
// this method is used by "New connection" wizard
// for example (IBX):
// Params.Text := 'user_name=' + Login + #13#10 + 'password=' + Password;
end;


{ TfrxSQLMemTable }

constructor TfrxSQLMemTable.Create(AOwner: TComponent);
begin
  FTable := TSQLMemTable.Create(nil);
  DataSet := FTable;
  SetDatabase(nil);
  inherited;
end;

constructor TfrxSQLMemTable.DesignCreate(AOwner: TComponent; Flags: Word);
var
  i: Integer;
  l: TList;
begin
  inherited;
  l := Report.AllObjects;
  for i := 0 to l.Count - 1 do
    if TObject(l[i]) is TfrxSQLMemDatabase then
    begin
      SetDatabase(TfrxSQLMemDatabase(l[i]));
      break;
    end;
end;

class function TfrxSQLMemTable.GetDescription: String;
begin
  Result := 'SMT Table';
end;

procedure TfrxSQLMemTable.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) and (AComponent = FDatabase) then
    SetDatabase(nil);
end;

procedure TfrxSQLMemTable.SetDatabase(const Value: TfrxSQLMemDatabase);
begin
  FDatabase := Value;
  if Value <> nil then
    FTable.DatabaseName := Value.DatabaseName
  else if SMTComponents <> nil then
    FTable.DatabaseName := SMTComponents.DefaultDatabase.DatabaseName
  else
    FTable.DatabaseName := '';
end;

function TfrxSQLMemTable.GetIndexFieldNames: String;
begin
  Result := FTable.IndexFieldNames;
end;

function TfrxSQLMemTable.GetIndexName: String;
begin
  Result := FTable.IndexName;
end;

function TfrxSQLMemTable.GetTableName: String;
begin
  Result := FTable.TableName;
end;

procedure TfrxSQLMemTable.SetIndexFieldNames(const Value: String);
begin
  FTable.IndexFieldNames := Value;
end;

procedure TfrxSQLMemTable.SetIndexName(const Value: String);
begin
  FTable.IndexName := Value;
end;

procedure TfrxSQLMemTable.SetTableName(const Value: String);
begin
  FTable.TableName := Value;
end;

procedure TfrxSQLMemTable.SetMaster(const Value: TDataSource);
begin
  FTable.MasterSource := Value;
end;

procedure TfrxSQLMemTable.SetMasterFields(const Value: String);
begin
  FTable.MasterFields := Value;
end;

procedure TfrxSQLMemTable.BeforeStartReport;
begin
  SetDatabase(FDatabase);
end;


{ TfrxSQLMemQuery }

constructor TfrxSQLMemQuery.Create(AOwner: TComponent);
begin
  FQuery := TSQLMemQuery.Create(nil);
  Dataset := FQuery;
  SetDatabase(nil);
  inherited;
end;

constructor TfrxSQLMemQuery.DesignCreate(AOwner: TComponent; Flags: Word);
var
  i: Integer;
  l: TList;
begin
  inherited;
  l := Report.AllObjects;
  for i := 0 to l.Count - 1 do
    if TObject(l[i]) is TfrxSQLMemDatabase then
    begin
      SetDatabase(TfrxSQLMemDatabase(l[i]));
      break;
    end;
end;

class function TfrxSQLMemQuery.GetDescription: String;
begin
  Result := 'SMT Query';
end;

procedure TfrxSQLMemQuery.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) and (AComponent = FDatabase) then
    SetDatabase(nil);
end;

procedure TfrxSQLMemQuery.SetDatabase(const Value: TfrxSQLMemDatabase);
begin
  FDatabase := Value;
  if Value <> nil then
    FQuery.DatabaseName := Value.DatabaseName
  else if SMTComponents <> nil then
    FQuery.DatabaseName := SMTComponents.DefaultDatabase.DatabaseName
  else
    FQuery.DatabaseName := '';
end;

function TfrxSQLMemQuery.GetSQL: TStrings;
begin
  Result := FQuery.SQL;
end;

procedure TfrxSQLMemQuery.SetSQL(Value: TStrings);
begin
  FQuery.SQL := Value;
end;

procedure TfrxSQLMemQuery.SetMaster(const Value: TDataSource);
begin
  FQuery.DataSource := Value;
end;

procedure TfrxSQLMemQuery.UpdateParams;
begin
  frxParamsToTParams(Self, FQuery.Params);
end;

procedure TfrxSQLMemQuery.BeforeStartReport;
begin
  SetDatabase(FDatabase);
end;

{$IFDEF QBUILDER}
function TfrxSQLMemQuery.QBEngine: TfqbEngine;
begin
  Result := TfrxEngineSMT.Create(nil);
  TfrxEngineSMT(Result).FQuery.Database := FQuery.Database;
end;
{$ENDIF}


{$IFDEF QBUILDER}
constructor TfrxEngineSMT.Create(AOwner: TComponent);
begin
  inherited;
  FQuery := TSQLMemQuery.Create(Self);
end;

destructor TfrxEngineSMT.Destroy;
begin
  FQuery.Free;
  inherited;
end;

procedure TfrxEngineSMT.ReadFieldList(const ATableName: string;
  var AFieldList: TfqbFieldList);
var
  TempTable: TSQLMemTable;
  Fields: TFieldDefs;
  i: Integer;
  tmpField: TfqbField;
begin
  AFieldList.Clear;
  TempTable := TSQLMemTable.Create(Self);
  TempTable.Database := FQuery.Database;
  TempTable.TableName := ATableName;
  Fields := TempTable.FieldDefs;
  try
    try
      TempTable.Active := True;
      tmpField:= TfqbField(AFieldList.Add);
      tmpField.FieldName := '*';
      for i := 0 to Fields.Count - 1 do
      begin
        tmpField := TfqbField(AFieldList.Add);
        tmpField.FieldName := Fields.Items[i].Name;
        tmpField.FieldType := Ord(Fields.Items[i].DataType)
      end;
    except
    end;
  finally
    TempTable.Free;
  end;
end;

procedure TfrxEngineSMT.ReadTableList(ATableList: TStrings);
begin
  ATableList.Clear;
  FQuery.Database.GetTableNames(ATableList, ShowSystemTables);
end;

function TfrxEngineSMT.ResultDataSet: TDataSet;
begin
  Result := FQuery;
end;

procedure TfrxEngineSMT.SetSQL(const Value: string);
begin
  FQuery.SQL.Text := Value;
end;
{$ENDIF}


initialization
  frxObjects.RegisterObject1(TfrxSQLMemDataBase, nil, 'Accuracer Database', 'SMT', 0, 37);
  frxObjects.RegisterObject1(TfrxSQLMemTable, nil, 'Accuracer Table', 'SMT', 0, 38);
  frxObjects.RegisterObject1(TfrxSQLMemQuery, nil, 'Accuracer Query', 'SMT', 0, 39);

finalization
//  CatBmp.Free;
  frxObjects.UnRegister(TfrxSQLMemDataBase);
  frxObjects.UnRegister(TfrxSQLMemTable);
  frxObjects.UnRegister(TfrxSQLMemQuery);


end.